@@ -4,7 +4,8 @@
 //! it which allows it to make the assumption the bytes are always returned with
 //! the `'de` lifetime.
 
-use core::fmt;
+use core::{fmt, slice};
+use std::{marker, ops::Range, ptr};
 
 use musli::error::Error;
 
@@ -58,6 +59,17 @@ pub trait Reader<'de> {
             reader: self,
         }
     }
+
+    /// Keep an accurate record of the position within the reader.
+    fn limit(self, limit: usize) -> Limit<Self>
+    where
+        Self: Sized,
+    {
+        Limit {
+            remaining: limit,
+            reader: self,
+        }
+    }
 }
 
 decl_message_repr!(SliceReaderErrorRepr, "error reading from slice");
@@ -173,6 +185,72 @@ where
     }
 }
 
+/// An efficient [Reader] wrapper around a slice.
+pub struct SliceReader<'de> {
+    range: Range<*const u8>,
+    _marker: marker::PhantomData<&'de [u8]>,
+}
+
+impl<'de> SliceReader<'de> {
+    /// Construct a new instance around the specified slice.
+    #[inline]
+    pub fn new(slice: &'de [u8]) -> Self {
+        Self {
+            range: slice.as_ptr_range(),
+            _marker: marker::PhantomData,
+        }
+    }
+}
+
+impl<'de> Reader<'de> for SliceReader<'de> {
+    type Error = SliceReaderError;
+
+    #[inline]
+    fn pos(&self) -> Option<usize> {
+        None
+    }
+
+    #[inline]
+    fn skip(&mut self, n: usize) -> Result<(), Self::Error> {
+        self.range.start = bounds_check_add(&self.range, n)?;
+        Ok(())
+    }
+
+    #[inline]
+    fn read_bytes(&mut self, n: usize) -> Result<&'de [u8], Self::Error> {
+        let outcome = bounds_check_add(&self.range, n)?;
+
+        unsafe {
+            let bytes = slice::from_raw_parts(self.range.start, n);
+            self.range.start = outcome;
+            Ok(bytes)
+        }
+    }
+
+    #[inline]
+    fn read(&mut self, buf: &mut [u8]) -> Result<(), Self::Error> {
+        let outcome = bounds_check_add(&self.range, buf.len())?;
+
+        unsafe {
+            ptr::copy_nonoverlapping(self.range.start, buf.as_mut_ptr(), buf.len());
+            self.range.start = outcome;
+        }
+
+        Ok(())
+    }
+}
+
+#[inline]
+fn bounds_check_add(range: &Range<*const u8>, len: usize) -> Result<*const u8, SliceReaderError> {
+    let outcome = range.start.wrapping_add(len);
+
+    if outcome > range.end || outcome < range.start {
+        Err(SliceReaderError::custom("buffer underflow"))
+    } else {
+        Ok(outcome)
+    }
+}
+
 /// Keep a record of the current position.
 ///
 /// Constructed through [Reader::with_position].
@@ -227,3 +305,68 @@ where
         Ok(array)
     }
 }
+
+/// Limit the number of bytes that can be read out of a reader to the specified limit.
+///
+/// Constructed through [Reader::limit].
+pub struct Limit<R> {
+    remaining: usize,
+    reader: R,
+}
+
+impl<'de, R> Limit<R>
+where
+    R: Reader<'de>,
+{
+    fn bounds_check(&mut self, n: usize) -> Result<(), R::Error> {
+        match self.remaining.checked_sub(n) {
+            Some(remaining) => {
+                self.remaining = remaining;
+                Ok(())
+            }
+            None => Err(R::Error::custom("out of bounds")),
+        }
+    }
+}
+
+impl<'de, R> Reader<'de> for Limit<R>
+where
+    R: Reader<'de>,
+{
+    type Error = R::Error;
+
+    #[inline]
+    fn pos(&self) -> Option<usize> {
+        self.reader.pos()
+    }
+
+    #[inline]
+    fn skip(&mut self, n: usize) -> Result<(), Self::Error> {
+        self.bounds_check(n)?;
+        self.reader.skip(n)
+    }
+
+    #[inline]
+    fn read_bytes(&mut self, n: usize) -> Result<&'de [u8], Self::Error> {
+        self.bounds_check(n)?;
+        self.reader.read_bytes(n)
+    }
+
+    #[inline]
+    fn read(&mut self, buf: &mut [u8]) -> Result<(), Self::Error> {
+        self.bounds_check(buf.len())?;
+        self.reader.read(buf)
+    }
+
+    #[inline]
+    fn read_byte(&mut self) -> Result<u8, Self::Error> {
+        self.bounds_check(1)?;
+        self.reader.read_byte()
+    }
+
+    #[inline]
+    fn read_array<const N: usize>(&mut self) -> Result<[u8; N], Self::Error> {
+        self.bounds_check(N)?;
+        self.reader.read_array()
+    }
+}
